"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const policies_1 = require("./policies");
class HTTPHeader {
    constructor(header, value, info = "N/A") {
        this.header = header;
        this.value = value;
        this.info = info;
    }
}
class SecurityHeaders {
    static secureHeaders(options) {
        let headers = [];
        options = utils_1.parseHeaderOptions(options);
        if (options.server) {
            if (options.server.option === policies_1.PolicyOptions.CustomValue &&
                options.server.value) {
                SecurityHeaders.server.value = options.server.value;
                headers.push(SecurityHeaders.server);
            }
            else if (options.server.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.server);
            }
        }
        if (options.hsts) {
            if (options.hsts.option === policies_1.PolicyOptions.CustomValue &&
                options.hsts.value) {
                SecurityHeaders.httpStrictTransportSecurity.value = options.hsts.value;
                headers.push(SecurityHeaders.httpStrictTransportSecurity);
            }
            else if (options.hsts.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.httpStrictTransportSecurity);
            }
        }
        if (options.xfo) {
            if (options.xfo.option === policies_1.PolicyOptions.CustomValue &&
                options.xfo.value) {
                SecurityHeaders.xFrameOptions.value = options.xfo.value;
                headers.push(SecurityHeaders.xFrameOptions);
            }
            else if (options.xfo.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.xFrameOptions);
            }
        }
        if (options.xxp) {
            if (options.xxp.option === policies_1.PolicyOptions.CustomValue &&
                options.xxp.value) {
                SecurityHeaders.xXssProtection.value = options.xxp.value;
                headers.push(SecurityHeaders.xXssProtection);
            }
            else if (options.xxp.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.xXssProtection);
            }
        }
        if (options.content) {
            if (options.content.option === policies_1.PolicyOptions.CustomValue &&
                options.content.value) {
                SecurityHeaders.xContentTypeOptions.value = options.content.value;
                headers.push(SecurityHeaders.xContentTypeOptions);
            }
            else if (options.content.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.xContentTypeOptions);
            }
        }
        if (options.csp) {
            if (options.csp.option === policies_1.PolicyOptions.CustomValue &&
                options.csp.value) {
                SecurityHeaders.contentSecurityPolicy.value = options.csp.value;
                headers.push(SecurityHeaders.contentSecurityPolicy);
            }
            else if (options.csp.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.contentSecurityPolicy);
            }
        }
        if (options.referrer) {
            if (options.referrer.option === policies_1.PolicyOptions.CustomValue &&
                options.referrer.value) {
                SecurityHeaders.referrerPolicy.value = options.referrer.value;
                headers.push(SecurityHeaders.referrerPolicy);
            }
            else if (options.referrer.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.referrerPolicy);
            }
        }
        if (options.cache) {
            if (options.cache.option === policies_1.PolicyOptions.CustomValue &&
                options.cache.value) {
                SecurityHeaders.cacheControl.value = options.cache.value;
                headers.push(SecurityHeaders.cacheControl);
            }
            else if (options.cache.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.cacheControl);
                headers.push(SecurityHeaders.pragma);
                headers.push(SecurityHeaders.expires);
            }
        }
        if (options.feature) {
            if (options.feature.option === policies_1.PolicyOptions.CustomValue &&
                options.feature.value) {
                SecurityHeaders.featurePolicy.value = options.feature.value;
                headers.push(SecurityHeaders.featurePolicy);
            }
            else if (options.feature.option === policies_1.PolicyOptions.DefaultHeader) {
                headers.push(SecurityHeaders.featurePolicy);
            }
        }
        return headers;
    }
}
SecurityHeaders.server = new HTTPHeader("Server", "NULL");
SecurityHeaders.httpStrictTransportSecurity = new HTTPHeader("Strict-Transport-Security", "max-age=63072000; includeSubdomains", "Ensure application communication is sent over HTTPS");
SecurityHeaders.xFrameOptions = new HTTPHeader("X-Frame-Options", "SAMEORIGIN", "Disable framing from different origins (clickjacking defense)");
SecurityHeaders.xXssProtection = new HTTPHeader("X-XSS-Protection", "1; mode=block", "Enable Cross-Site Scripting filters");
SecurityHeaders.xContentTypeOptions = new HTTPHeader("X-Content-Type-Options", "nosniff", "Prevent MIME-sniffing");
SecurityHeaders.contentSecurityPolicy = new HTTPHeader("Content-Security-Policy", "script-src 'self'; object-src 'self'", "Prevent Cross-site injections");
SecurityHeaders.referrerPolicy = new HTTPHeader("Referrer-Policy", "no-referrer, strict-origin-when-cross-origin", "Enable full referrer if same origin, remove path for cross origin and disable referrer in unsupported browsers");
SecurityHeaders.cacheControl = new HTTPHeader("Cache-control", "no-cache, no-store, must-revalidate, max-age=0", "Prevent cacheable HTTPS response");
SecurityHeaders.pragma = new HTTPHeader("Pragma", "no-cache", "Prevent cacheable HTTPS response");
SecurityHeaders.expires = new HTTPHeader("Expires", "0", "Prevent cacheable HTTPS response");
SecurityHeaders.featurePolicy = new HTTPHeader("Feature-Policy", "accelerometer 'none'; ambient-light-sensor 'none'; autoplay 'none'; camera 'none'; encrypted-media 'none'; fullscreen 'none'; geolocation 'none'; gyroscope 'none'; magnetometer 'none'; microphone 'none'; midi 'none'; payment 'none'; picture-in-picture 'none'; speaker 'none'; sync-xhr 'none'; usb 'none'; vr 'none'", "Disable browser features and APIs");
exports.SecurityHeaders = SecurityHeaders;
function headerObject(options) {
    const headers = SecurityHeaders.secureHeaders(options);
    let secureHeaders = {};
    for (let header of headers) {
        secureHeaders[header.header] = header.value;
    }
    return secureHeaders;
}
exports.headerObject = headerObject;
function setCommonHeader(res, options) {
    const headers = headerObject(options);
    res.set(headers);
}
exports.setCommonHeader = setCommonHeader;
function setHeaderTuple(response, options) {
    const headers = SecurityHeaders.secureHeaders(options);
    for (let header of headers) {
        response.header(header.header, header.value);
    }
}
exports.setHeaderTuple = setHeaderTuple;
function setHeader(res, options) {
    const headers = SecurityHeaders.secureHeaders(options);
    for (let header of headers) {
        res.setHeader(header.header, header.value);
    }
}
exports.setHeader = setHeader;
